<?php
// (c) Copyright 2002-2013 by authors of the Tiki Wiki CMS Groupware Project
// 
// All Rights Reserved. See copyright.txt for details and a complete list of authors.
// Licensed under the GNU LESSER GENERAL PUBLIC LICENSE. See license.txt for details.
// $Id: Location.php 46420 2013-06-19 19:33:43Z arildb $

/**
 * Handler class for location/map/gmap
 * 
 * Letter key: ~G~
 *
 */
class Tracker_Field_Location extends Tracker_Field_Abstract implements Tracker_Field_Synchronizable
{
	public static function getTypes()
	{
		return array(
			'G' => array(
				'name' => tr('Location'),
				'description' => tr('Allows to select a geolocation for the item and displays it on a map.'),
				'help' => 'Location Tracker Field',				
				'prefs' => array('trackerfield_location'),
				'tags' => array('basic'),
				'default' => 'y',
				'params' => array(
					'use_as_item_location' => array(
						'name' => tr('Use as item location'),
						'description' => tr('When enabled, records the field\'s value as the item\'s geolocation to be displayed on locator maps.'),
						'filter' => 'int',
						'options' => array(
							0 => tr('No'),
							1 => tr('Yes'),
						),
					),
					'list_width' => array(
						'name' => tr('List View Width'),
						'description' => tr('Width of map in pixels when tracker items is shown in list view'),
						'filter' => 'int',
						'default' => 200,
					),
					'list_height' => array(
						'name' => tr('List View Height'),
						'description' => tr('Height of map in pixels when tracker items is shown in list view'),
						'filter' => 'int',
						'default' => 200,
					),
					'item_width' => array(
						'name' => tr('Item View Width'),
						'description' => tr('Width of map in pixels when single tracker items is shown'),
						'filter' => 'int',
						'default' => 500,
					),
					'item_height' => array(
						'name' => tr('Item View Height'),
						'description' => tr('Height of map in pixels when single tracker items is shown'),
						'filter' => 'int',
						'default' => 400,
					),
				),
			),
		);
	}

	function getFieldData(array $requestData = array())
	{
		if (isset($requestData[$this->getInsertId()])) {
			$value = $requestData[$this->getInsertId()];
		} else {
			$value = $this->getValue();
		}
		
		$parts = explode(',', $value);
		$parts = array_map('floatval', $parts);

		if (count($parts) >= 2) {
			// Always use . as the decimal point in the value, and not comma as used some places
			$value = '';
			$value .= str_replace(',', '.', $parts[0]).',';
			$value .= str_replace(',', '.', $parts[1]).',';
			$value .= str_replace(',', '.', $parts[2]);

			return array(
				'value' => $value,
				'x' => $parts[0],	
				'y' => $parts[1],
				'z' => isset($parts[2]) ? $parts[2] : 0,
			);
		} else {
			return array(
				'value' => '',
				'x' => null,
				'y' => null,
				'z' => null,
			);
		}
	}

	function renderInput($context = array())
	{
		TikiLib::lib('header')->add_map();
		return $this->renderTemplate('trackerinput/location.tpl', $context);
	}

	function renderOutput($context = array())
	{
		if ($context['list_mode'] === 'csv') {
			return $this->getConfiguration('value');
		} else {
			TikiLib::lib('header')->add_map();
			return $this->renderTemplate('trackeroutput/location.tpl', $context);
		}
	}

	function importRemote($value)
	{
		return $value;
	}

	function exportRemote($value)
	{
		return $value;
	}

	function importRemoteField(array $info, array $syncInfo)
	{
		return $info;
	}
}

